#!/usr/bin/python

import gobject

import sys
sys.path.append("/usr/share/sectool")

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop

import output
import actions
from sectool import TuiOptions

import socket

try:
    # try to import the module installed in the system
    import slip.dbus.service
except ImportError:
    # try to find the module in the unpacked source tree
    import os.path
    import import_marker

    # try to find the slip.dbus module

    _modfile = import_marker.__file__
    _path = os.path.dirname (_modfile)
    _found = False
    _oldsyspath = sys.path
    while not _found and _path and _path != "/":
        _path = os.path.abspath (os.path.join (path, os.path.pardir))
        sys.path = _oldsyspath + [_path]
        try:
            import slip.dbus.service
            _found = True
        except ImportError:
            pass
    if not _found:
        import slip.dbus.service
    sys.path = _oldsyspath


class SectoolObject(slip.dbus.service.Object):

    def __init__ (self):
        DBusGMainLoop(set_as_default=True)
        bus = dbus.SystemBus()
        bus_name = dbus.service.BusName('org.fedoraproject.sectool.mechanism', bus)
        super (SectoolObject, self).__init__ (bus, '/org/fedoraproject/sectool/object')

    def run(self):
        mainloop = gobject.MainLoop()
        slip.dbus.service.set_mainloop (mainloop)
        mainloop.run()

    def __parse_options( self, args ):

        sys.argv = args
        options = TuiOptions()
        options.parse_params(args)
        options.loadSchedulerOptions()
        return options

    def __perform_action(self, constructor, options, file):

        # generate unique hash to make unique file as output pipe
        # TODO: rewrite this by using tmpfile library

        self.output = FileOutput( file )
        fmt = output.StdoutFormatter( self.output )
        action = constructor( options, fmt, self.output )
        action.doAction()
        self.output.close()
        

    def ST_Action(self, action, args, file):

        args = [str(i) for i in args]
        options = self.__parse_options( args )

        # perform action
        return self.__perform_action( action, options, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.listtests")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def listtests(self, args, file):
        
        return self.ST_Action( actions.ST_ListTests, args, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.runlevel")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def runlevel(self, args, file):

        return self.ST_Action( actions.ST_RunLevel, args, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.runtests")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def runtests(self, args, file):

        return self.ST_Action( actions.ST_RunTests, args, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.infotests")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def infotests(self, args, file):

        return self.ST_Action( actions.ST_InfoTests, args, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.auto")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def auto(self, args, file):

        return self.ST_Action( actions.ST_Auto, args, file )

    @slip.dbus.polkit.require_auth ("org.fedoraproject.sectool.cleancache")
    @dbus.service.method("org.fedoraproject.sectool.mechanism", in_signature='ass')
    def cleancache(self, args, file):

        return self.ST_Action( actions.ST_CleanCache, args, file )

class FileOutput:

    def __init__(self, file):
        self.path = file
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.socket.connect(self.path)

    def write(self, s):
        self.socket.send(s)

    def isatty(self):
        return False

    def close(self):
        self.write("#END_FILE")
        self.socket.close()

if __name__ == '__main__':

    obj = SectoolObject()
    obj.run()
